<#
.SYNOPSIS
  Clean up files and folders

.DESCRIPTION
  Remove files older than X days
  Delete all empty folders
  Archives files between X and Y days
.PARAMETER <Parameter_Name>
  
.INPUTS
  None
.OUTPUTS Log File
  None
.NOTES
  Original by Terri Donahue
  Refactored by Scott Forsyth 10/5/21

  Version:        2.0
  Author:         Terri Donahue / Arianna Lawson / Scott Forsyth
  Creation Date:  ~2017
  Update Date:    10/5/2021
  Purpose/Change: Added governor, refactored, testing and fixes
  Update Date:    8/30/22
  Author:         Scott Forsyth
  Purpose/Change: Added Remove-StaleDiagnosticsFolders
.EXAMPLE
  
  
#>
#---------------------------------------------------------[Script Parameters]------------------------------------------------------

#----------------------------------------------------------[Declarations]----------------------------------------------------------

$dataDrive = "G:"
$siteLocation   = Join-Path $dataDrive '\domains\sites\'
$backupLoc      = Join-Path $dataDrive '\logsbackups'

#tweak the following to get a good balance between high CPU and how long the script runs'
#initial testing on desktop suggests that 0 will have near 100CPU, and 1ms will drop to 20%
$throttleBackupsInMilliseconds = 1
$throttleDeletesInMilliseconds = 1
$throttleEmptyFolderDeletesInMilliseconds = 1

$SetPaths={
    # [Path suffix] [Days to retain backup] [Days to retain before deleting]
    # To not have backups run, set the days to 0
    Add-ToPathList '\files\system\log\dataintegration\'         0 7
    Add-ToPathList '\files\system\log\scheduledtasks\'          0 7
    Add-ToPathList '\files\system\diagnostics\' 	            0 7
    Add-ToPathList '\files\system\log\eventviewer\'             0 7
    Add-ToPathList '\files\system\log\templates\'               0 7
	
    Add-ToPathList '\files\System\Log\eCom\Frontend\'           0 10
	Add-ToPathList '\files\System\Log\Punchout\'                0 30
	Add-ToPathList '\files\System\Log\LessCompiler\'            0 30
    Add-ToPathList '\files\System\Log\BioLegend.IPLookup\'      0 14
    Add-ToPathList '\files\System\Log\Dna\' 				    0 14
    Add-ToPathList '\files\System\Log\DynamicwebNA\'            0 14
    Add-ToPathList '\files\System\Log\IpBanner\'                0 14
	Add-ToPathList '\files\System\Log\'                         0 30

}
$foldersList = New-Object System.Collections.ArrayList
$sites       = Get-ChildItem -Directory $siteLocation

#New 8/30/22
$StatusXmlFolderRetentionAmount = 3

#-----------------------------------------------------------[Functions]------------------------------------------------------------
function Add-ToPathList{
    param (
        $pathSuffix, 
        $daysToRetainBackups,
        $daysToRetainOriginal
    )

    $paths = [PSCustomObject]@{'PathSuffix'=$pathSuffix;'DaysToRetainBackups'=$daysToRetainBackups;'DaysToRetainOriginal'=$daysToRetainOriginal}
    $foldersList.Add($paths) | Out-Null
}


function Run-Backups{
    param(
        $pathRoot,
        $pathSuffix,
        $daysToRetainOriginal,
        $daysToRetainBackups
        )
    $path = Join-Path $pathRoot $pathSuffix

    Write-Log "Processing backups for $path"

    if (!(Test-Path $path)){
        return
    }

    $logs = Get-ChildItem -Recurse -Path $path -File -Filter *.log | Where-Object -FilterScript {
        ($_.LastWriteTime -ge (Get-Date).AddDays(-1 * $daysToRetainBackups) -and $_.LastWriteTime -le (Get-Date).AddDays(-1 * $daysToRetainOriginal)) -or ($_.Length -eq 0KB)
    }
    
    $backupFolder = $path.Replace($dataDrive, $backupLoc)
    if ($logs){
        If (!(Test-Path -Path $backupFolder -PathType Container))
        {
            New-Item -Path $backupFolder -ItemType directory -Force
        }
    }
    
    foreach ($log in $logs) 
    {
        Move-Item -Path $log.FullName -Destination $backupFolder -Force
        Write-Log "Archiving $($log.FullName) with timestamp of $($log.LastWriteTime)"
        Start-Sleep -Milliseconds $throttleBackupsInMilliseconds
    }
}

function Run-RemoveOldBackups{
    param(
        $pathRoot,
        $pathSuffix,
        $daysToRetainBackups
        )
        
    $path = Join-Path $pathRoot $pathSuffix
    $backupFolder = $path.Replace($dataDrive, $backupLoc)

    Write-Log "Processing removing of old backups for $backupFolder"

    if (!(Test-Path $backupFolder)){
        return
    }

    $logs = Get-ChildItem -Recurse -Path $backupFolder -File -Filter *.log | Where-Object -FilterScript {
        $_.LastWriteTime -lt (Get-Date).AddDays(-1 * $daysToRetainBackups) 
    }
    
    foreach ($log in $logs) 
    {
        Remove-Item -Path $log.FullName
        Write-Log "Deleting old backup $($log.FullName) with timestamp of $($log.LastWriteTime)"

        Start-Sleep -Milliseconds $throttleBackupsInMilliseconds
    }

    Remove-EmptyFolders $backupFolder
}

function Remove-EmptyFolders{
    Param(
        $path
    )

    if (!(Test-Path $path)){
        return
    }

    Get-ChildItem -Path $path -Directory -Recurse | ForEach-Object {
        $folder = $_
        $fileCount = ($folder | Get-ChildItem | Measure-Object).count
        if ($fileCount -eq 0){
            $folder | Remove-Item
            Write-Log "Removing empty folder $($folder)"
        }
        Start-Sleep -Milliseconds $throttleEmptyFolderDeletesInMilliseconds
    }
}

function Remove-StaleDiagnosticsFolders{
    #handle Diagnostics status.xml file
    if ($StatusXmlFolderRetentionAmount -and $StatusXmlFolderRetentionAmount -gt 0){
        #get Diagnostics folder path
        $diagnosticsPath = Join-Path $site.FullName '\files\System\Diagnostics'
        if (Test-Path $diagnosticsPath){
            #loop through each repository folder in the Diasnostics folder
            $repositoryFolders = Get-ChildItem -Path $diagnosticsPath -Directory
            foreach ($repositoryFolder in $repositoryFolders){
                #loop through each index folder in the Diagnostics folder
                $repositorySubfolders = Get-ChildItem -Path $repositoryFolder.FullName -Directory
                foreach ($repositorySubfolder in $repositorySubfolders){
                    # index subfolders
                    if ($repositorySubfolder.Name.EndsWith('.index')){
                        #loop through each instance folder in the index folder
                        $instanceFolders = Get-ChildItem -Path $repositorySubfolder.FullName -Directory
                        foreach ($instanceFolder in $instanceFolders){
                            #get the instance subfolders, except not the first $StatusXmlFolderRetentionAmount. Due to the naming pattern, we can sort by name descending (the date modified is not accurate)
                            $instanceSubFolders = Get-ChildItem -Path $instanceFolder.FullName -Directory | `
                                    Sort-Object -Property Name -Desc | `
                                    Select-Object -Skip $StatusXmlFolderRetentionAmount
                            foreach ($instanceSubFolder in $instanceSubFolders){
                                Write-Host "Removing folder $($instanceSubFolder.FullName)"
                                Remove-Item -Path $instanceSubFolder.FullName -Force -Recurse
                            }
                        }
                    }
                    #task subfolders
                    elseif ($repositorySubfolder.Name.EndsWith('.task')){
                        $instanceSubFolders = Get-ChildItem -Path $repositorySubfolder.FullName -Directory | `
                                Sort-Object -Property Name -Desc | `
                                Select-Object -Skip $StatusXmlFolderRetentionAmount
                        foreach ($instanceSubFolder in $instanceSubFolders){
                            Write-Host "Removing folder $($instanceSubFolder.FullName)"
                            Remove-Item -Path $instanceSubFolder.FullName -Force -Recurse
                        }
                    }
                }
            }
        }
    }
}

function Run-Deletes{
    param(
        $pathRoot,
        $pathSuffix,
        [int]$daysToRetain
    )

    $path = Join-Path $pathRoot $pathSuffix
    Write-Log "Processing deletes for $path"

    if (!(Test-Path $path)){
        return
    }

    $logs = Get-ChildItem -Recurse -Path $path -File -Filter *.log | Where-Object {
        ($_.LastWriteTime -lt (Get-Date).AddDays(-1 * $daysToRetain)) -or ($_.Length -eq 0KB)
    }
    
    foreach ($log in $logs) 
    {
        Write-Log "Deleting $($log.FullName) with timestamp of $($log.LastWriteTime)"
        Remove-Item $log.FullName
        Start-Sleep -Milliseconds $throttleDeletesInMilliseconds
    }

    Remove-EmptyFolders $path
}

function Write-Log{
    Param($str)
    Write-Host $str 
}

#-----------------------------------------------------------[Execution]------------------------------------------------------------


#Run SetPaths (defined at the top of the file)
& $SetPaths

foreach ($site in $sites) 
{
    Write-Log "Processing $($site.Name)"

    #handle all .log files
    foreach ($folder in $foldersList){
       Run-RemoveOldBackups $site.FullName          $folder.PathSuffix      $folder.DaysToRetainBackups
       Run-Backups          $site.FullName          $folder.PathSuffix      $folder.DaysToRetainOriginal       $folder.DaysToRetainBackups
       Run-Deletes          $site.FullName          $folder.PathSuffix      $folder.DaysToRetainOriginal
    }
    
    Remove-StaleDiagnosticsFolders
}
